@@ -3,6 +3,7 @@ open Result.Syntax
 
 (* OCaml library names *)
 
+let b0_std = B0_ocaml.libname "b0.std"
 let compiler_libs_toplevel = B0_ocaml.libname "compiler-libs.toplevel"
 let cmdliner = B0_ocaml.libname "cmdliner"
 let unix = B0_ocaml.libname "unix"
@@ -18,21 +19,22 @@ let fmt_lib =
   B0_ocaml.lib fmt ~srcs:[`Dir ~/"src"]
 
 let fmt_cli =
-  B0_ocaml.lib fmt_cli ~srcs:[`Dir ~/"src/cli"] ~requires:[cmdliner; fmt]
+  let srcs = [`Dir ~/"src/cli"] in
+  B0_ocaml.lib fmt_cli ~srcs ~requires:[cmdliner; fmt] ~exports:[fmt]
 
 let fmt_tty =
-  B0_ocaml.lib fmt_tty ~srcs:[`Dir ~/"src/tty"] ~requires:[unix; fmt]
+  let srcs = [`Dir ~/"src/tty"] in
+  B0_ocaml.lib fmt_tty ~srcs ~requires:[unix; fmt] ~exports:[fmt]
 
 let fmt_top =
-  let requires = [compiler_libs_toplevel] in
   let srcs = [`Dir ~/"src/top"; `X ~/"src/top/fmt_tty_top_init.ml"] in
-  B0_ocaml.lib fmt_top ~srcs ~requires
+  B0_ocaml.lib fmt_top ~srcs ~requires:[compiler_libs_toplevel]
 
 (* Tests *)
 
 let test ?(requires = []) = B0_ocaml.test ~requires:(fmt :: requires)
 
-let test_fmt = test ~/"test/test_fmt.ml"
+let test_fmt = test ~/"test/test_fmt.ml" ~requires:[b0_std]
 let styled_perf_bug =
   test ~/"test/styled_perf_bug.ml" ~requires:[unix] ~run:false
 
@@ -59,7 +61,7 @@ let default =
       [ "ocaml", {|>= "4.08.0"|};
         "ocamlfind", {|build|};
         "ocamlbuild", {|build|};
-        "topkg", {|build & >= "1.0.3"|};
+        "topkg", {|build & >= "1.1.0"|};
       ]
     |> ~~ B0_opam.build
       {|[["ocaml" "pkg/pkg.ml" "build" "--dev-pkg" "%{dev}%"
